/* $Id: etherints.c,v 1.3 89/07/11 03:07:41 dupuy Exp $ */

/*
 * Returns an array of strings (with last element 0) each entry of which is an
 * ethernet interface name valid for use in ether_open().  Valid prefixes for
 * ethernet interface names are in validname.  Assumes that the device
 * prefixes are unique (i.e. no eckankar0 device exists).
 */

#include <strings.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#ifdef _SOCKET_				/* 4.0 system? */
#include <sys/sockio.h>
#else
#endif
#include <sys/ioctl.h>

static char checked;			/* boolean flag */

#define MAXNUMIF 8			/* maximum number of interfaces */

static char *result[MAXNUMIF];
static char names[MAXNUMIF * IFNAMSIZ];
#ifdef sun
static char *validname[] = {"le", "ie", "ec", 0};
#endif
#ifdef vax
static char *validname[] = {"de", "qe", "ni", "se",
			    "ec", "en", "ex", "il", "ix", 0};
#endif

char **
ether_interfaces ()
{
    char cbuf[MAXNUMIF * sizeof (struct ifreq)];
    struct ifconf ifc;
    int sfd;

    register int i, j, k;
    int numinter;

    if (checked)
	return (result);		/* we have alread done this */

    /* we have not yet checked the interfaces */

    if ((sfd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
    {
#ifdef DEBUG
	perror ("libether socket:");
#endif
	return (0);
    }

    ifc.ifc_len = sizeof (cbuf);
    ifc.ifc_buf = cbuf;
    if (ioctl (sfd, SIOCGIFCONF, (char *) &ifc) < 0)
    {
#ifdef DEBUG
	perror ("libether SIOCGIFCONF:");
#endif
	close (sfd);
	return (0);
    }

    k = 0;
    numinter = ifc.ifc_len / sizeof (struct ifreq);
    for (i = 0; i < numinter; ++i)
	for (j = 0; validname[j] != 0; ++j)
	    if (strncmp (ifc.ifc_req[i].ifr_name, validname[j], 2) == 0)
	    {
		strcpy (names + IFNAMSIZ * k, ifc.ifc_req[i].ifr_name);
		result[k] = names + IFNAMSIZ * k;
		++k;
	    }
    result[k] = 0;


    close (sfd);
    checked = 1;
    return (result);
}
